package com.starbuds.server.api.common.v1.facade;

import com.starbuds.server.common.pojo.api.Result;

import javax.ws.rs.FormParam;

/**
 * 登录相关接口
 *
 * @author Lay
 * @date 2019/11/29
 */
public interface LoginFacade {
    /**
     * 短信验证码登录
     *
     * @param phone
     * @param code
     * @param inviteCode
     * @param countryId
     * @param channelId
     * @return
     */
    Result bySmsCode(String phone, String code, String inviteCode, String refreshToken, Integer countryId, Long channelId);

    /**
     * h5短信验证登录
     *
     * @param phone
     * @param code
     * @param countryId
     * @return
     */
    Result h5BySmsCodeLogin(@FormParam("phone") String phone,
                            @FormParam("code") String code,
                            @FormParam("imageCode") String imageCode,
                            @FormParam("imagecodeMd5") String imagecodeMd5,
                            @FormParam("countryId") Integer countryId);

    /**
     * 获取图片验证码
     * @return
     */
    Result getImageVerifyCode();

    /**
     * 发送短信验证码
     *
     * @param phone
     * @param codeType
     * @param isDevSend
     * @param countryId
     * @param sessionId
     * @param sig
     * @param token
     * @param scene
     * @return
     */
    Result sendSmsCode(String phone,
                       int codeType,
                       boolean isDevSend,
                       Integer countryId,
                       String sessionId,
                       String sig,
                       String token,
                       String scene);

    /**
     * h5发送短信验证码
     *
     */
    Result sendH5SmsCode(@FormParam("phone") String phone,
                         @FormParam("codeType") int codeType,
                         @FormParam("countryId") Integer countryId,
                         @FormParam("imageCode") String imageCode,
                         @FormParam("imagecodeMd5") String imagecodeMd5,
                         @FormParam("isDevSend") boolean isDevSend
    );

    /**
     * 手机号一键登录
     *
     * @param cert
     * @param inviteCode
     * @return
     */
    Result byOneKey(String cert, String inviteCode, String refreshToken);

    /**
     * 第三方 OAuth 登录
     *
     * @param type
     * @param code
     * @param inviteCode
     * @return
     */
    Result byThirdPartyOAuth(int type, String code, String inviteCode);

    /**
     * 校验短信验证码
     *
     * @param codeType
     * @param phone
     * @param smsCode
     * @param countryId
     * @return
     */
    Result verifySmsCode(int codeType, String phone, String smsCode, Integer countryId);

    /**
     * 密码登录
     *
     * @param phone
     * @param password
     * @param countryId
     * @return
     */
    Result byPassword(String phone, String password, Integer countryId);

    /**
     * 设置密码
     *
     * @param password
     * @return
     */
    Result setPassword(String password);

    /**
     * 根据token获取个人资料
     *
     * @param token
     * @return
     */
    Result getProfileByToken(String token);

    /**
     * 获取 RTM 令牌
     *
     * @return
     */
    Result getRtmToken();


    /**
     * 激活
     *
     * @return
     */
    Result activation();


    /**
     * 根据用户ID获取用户是否实名认证
     *
     * @param userId
     * @return
     */
    Result getUserIsAuth(Long userId);

}
