package com.starbuds.server.api.common.v1.impl;

import com.starbuds.server.api.common.inf.PermitGuest;
import com.starbuds.server.api.common.v1.facade.DataFacade;
import com.starbuds.server.common.constant.common.AppType;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.data.AreaService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.FileService;
import com.starbuds.server.common.service.i18n.I18nService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import java.util.Optional;

/**
 * 数据接口实现
 *
 * @author Lay
 * @date 2019/12/23
 */
@Path("/v1/data")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class DataFacadeImpl extends BaseFacade implements DataFacade {

    @Reference(version = "1.0.0")
    FileService fileService;

    @Reference(version = "1.0.0")
    AreaService areaService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    private I18nService i18nService;

    /**
     * 获取用户头像上传配置
     *
     * @return
     */
    @GET
    @Path("getUserAvatarUploadConfig")
    @Override
    public Result getUserAvatarUploadConfig() {
        Long userId = this.getToken().getUserId();
        String key = fileService.getUserAvatarKey(userId);
        String upToken = fileService.getUserUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取用户封面上传配置
     *
     * @return
     */
    @GET
    @Path("getUserCoverUploadConfig")
    @Override
    public Result getUserCoverUploadConfig() {
        Long userId = this.getToken().getUserId();
        String key = fileService.getUserCoverKey(userId);
        String upToken = fileService.getUserUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取用户认证上传配置
     *
     * @return
     */
    @GET
    @Path("getUserAuthUploadConfig")
    @Override
    public Result getUserAuthUploadConfig() {
        Long userId = this.getToken().getUserId();
        String key = fileService.getUserAuthKey(userId);
        String upToken = fileService.getUserUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取举报图上传配置
     *
     * @param userId
     * @return
     */
    @GET
    @Path("getReportUploadConfig")
    @Override
    public Result getReportUploadConfig(@QueryParam("userId") Long userId) {
        String key = fileService.getReportKey(this.getToken().getUserId(), userId);
        String upToken = fileService.getUserUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取意见反馈图上传配置
     *
     * @return
     */
    @GET
    @Path("getSuggestUploadConfig")
    @Override
    public Result getSuggestUploadConfig() {
        String key = fileService.getSuggestKey(this.getToken().getUserId());
        String upToken = fileService.getUserUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取内容图片上传配置
     *
     * @return
     */
    @GET
    @Path("getFeedImageUploadConfig")
    @Override
    public Result getFeedImageUploadConfig() {
        Long userId = this.getToken().getUserId();
        String key = fileService.getFeedImageKey(userId);
        String upToken = fileService.getFeedUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取内容视频上传配置
     *
     * @return
     */
    @GET
    @Path("getFeedVideoUploadConfig")
    @Override
    public Result getFeedVideoUploadConfig() {
        Long userId = this.getToken().getUserId();
        String key = fileService.getFeedVideoKey(userId);
        String upToken = fileService.getFeedUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取内容视频封面上传配置
     *
     * @return
     */
    @GET
    @Path("getFeedVideoCoverUploadConfig")
    @Override
    public Result getFeedVideoCoverUploadConfig() {
        Long userId = this.getToken().getUserId();
        String key = fileService.getFeedVideoCoverKey(userId);
        String upToken = fileService.getFeedUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取区域列表
     *
     * @param parentId
     * @return
     */
    @GET
    @Path("getAreaList")
    @PermitGuest
    @Override
    public Result getAreaList(@QueryParam("parentId") int parentId) {
        MutableResult mResult = new MutableResult();
        mResult.put("list", areaService.getAreaList(parentId));
        return mResult.build();
    }

    /**
     * 获取意见反馈类型
     *
     * @return
     */
    @GET
    @Path("getSuggestTypeList")
    @PermitGuest
    @Override
    public Result getSuggestTypeList() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", i18nService.doI18n(dataService.getFeedbackTypeList(), lang()));
        return mResult.build();
    }

    /**
     * 获取应用最新版本
     *
     * @param versionNum
     * @return
     */
    @GET
    @Path("getAppVersionNew")
    @PermitGuest
    @Override
    public Result getAppVersionNew(@QueryParam("versionNum") long versionNum) {
        return new Result(dataService.getLatestAppVersion(this.getHeader().getClientType(), this.getHeader().getAppType(), versionNum, Optional.ofNullable(this.getHeader().getChannelId()).orElse(0L)));
    }

    /**
     * 获取最新应用信息
     *
     * @return
     */
    @GET
    @Path("getAppLatest")
    @PermitAll
    @Override
    public Result getAppLatest(@QueryParam("clientType") int clientType,
                               @QueryParam("appType") int appType) {
        ClientType clientTypeEnum = ClientType.toEnum(clientType);
        if (ClientType.UnKnow == clientTypeEnum) {
            return new Result(GlobalApiError.OperationFailed, "无效的客户端类型");
        }
        AppType appTypeEnum = AppType.toEnum(appType);
        if (appTypeEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "无效的应用类型");
        }
        Long channelId = 0L;
        if (this.getHeader() != null) {
            channelId = Optional.ofNullable(this.getHeader().getChannelId()).orElse(0L);
        }
        if (channelId.equals(0L)) {
            if (ClientType.Android.getValue() == clientType) {
                channelId = 3L;
            } else if (ClientType.iOS.getValue() == clientType) {
                channelId = 201L;
            }
        }
        return new Result(dataService.getLatestAppVersion(clientTypeEnum, appTypeEnum, 0, channelId));
    }

    /**
     * 获取当前时间
     *
     * @return
     */
    @GET
    @Path("getCurrentTime")
    @PermitAll
    @Override
    public Result getCurrentTime() {
        return new Result(System.currentTimeMillis());
    }

    /**
     * 获取国家区号
     *
     * @return
     */
    @GET
    @Path("getCountryPhoneCode")
    @PermitAll
    @Override
    public Result getCountryPhoneCode() {
        MutableResult mResult = new MutableResult();
        mResult.put("list", dataService.getPhoneAreaCode());
        return mResult.build();
    }

    /**
     * 获取房间封面上传配置
     *
     * @return
     */
    @GET
    @Path("getRtcRoomCoverUploadConfig")
    @PermitAll
    @Override
    public Result getRtcRoomCoverUploadConfig() {
        String key = fileService.getRtcRoomCoverKey();
        String upToken = fileService.getUserUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取音乐上传配置
     *
     * @return
     */
    @GET
    @Path("getMusicUploadConfig")
    @Override
    public Result getMusicUploadConfig() {

        Long userId = this.getToken().getUserId();
        String key = fileService.getMusicUploadKey(userId);
        String upToken = fileService.getFileUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取技能形象照上传配置
     *
     * @return
     */
    @GET
    @Path("getSkillImagePhotoUploadConfig")
    @Override
    public Result getSkillImagePhotoUploadConfig() {
        String key = fileService.getSkillImagePhotoUploadKey();
        String upToken = fileService.getFileUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }

    /**
     * 获取技能录音上传配置
     *
     * @return
     */
    @GET
    @Path("getSkillVoiceUploadConfig")
    @Override
    public Result getSkillVoiceUploadConfig() {
        String key = fileService.getSkillVoiceUploadKey();
        String upToken = fileService.getFileUploadToken(key, 300);

        MutableResult mResult = new MutableResult();
        mResult.put("upToken", upToken);
        mResult.put("key", key);
        return mResult.build();
    }
}
