package com.starbuds.server.api.common.v1.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.common.constant.ApiError;
import com.starbuds.server.api.common.inf.PermitGuest;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.data.GreenAuditStatusEnum;
import com.starbuds.server.common.constant.rocketmq.RocketTag;
import com.starbuds.server.common.constant.rocketmq.RocketTopic;
import com.starbuds.server.common.constant.user.*;
import com.starbuds.server.common.pojo.data.RiskParms;
import com.starbuds.server.common.pojo.data.green.GreenAuditTrace;
import com.starbuds.server.common.pojo.db.UserForbiddenDO;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.service.data.RiskService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.common.utils.ImageUtil;
import com.starbuds.server.common.utils.SafeUtil;
import com.starbuds.server.api.common.util.SpringContextUtil;
import com.starbuds.server.api.common.v1.facade.LoginFacade;
import com.starbuds.server.common.constant.activity.TriggerEventEnum;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.constant.common.*;
import com.starbuds.server.common.constant.message.sms.SmsActionEnum;
import com.starbuds.server.common.pojo.activity.UserLoginEventParam;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.message.SendResult;
import com.starbuds.server.common.pojo.user.*;
import com.starbuds.server.common.service.activity.TriggerService;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.message.SmsService;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.common.service.user.*;
import io.netty.util.internal.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.Base64Utils;

import javax.annotation.security.PermitAll;
import javax.imageio.ImageIO;
import javax.ws.rs.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import static com.starbuds.server.api.common.util.SpringContextUtil.isDev;

/**
 * 登录接口实现
 *
 * @author Lay
 * @date 2019/11/29
 */
@Path("/v1/login")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class LoginFacadeImpl extends BaseFacade implements LoginFacade {

    @Reference(version = "1.0.0")
    SmsService smsService;

    @Reference(version = "1.0.0")
    SecurityService securityService;

    @Reference(version = "1.0.0")
    UserService userService;

    @Reference(version = "1.0.0")
    ThirdAuthService thirdAuthService;

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    TriggerService triggerService;

    @Reference(version = "1.0.0")
    DeviceService deviceService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    ChannelService channelService;

    @Reference(version = "1.0.0")
    ThirdPartyService thirdPartyService;

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RiskService riskService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    private static final String AES_SECRET = "jovlvdpz2p53tpiy";

    private static final String API_AES_SECRET = "7pyah46cgjbt2vff303alsgx2ostd6of";

    /**
     * 短信验证码登录
     *
     * @param phone
     * @param code
     * @param inviteCode
     * @param countryId
     * @param channelId
     * @return
     */
    @POST
    @Path("bySmsCode")
    @PermitGuest
    @Override
    public Result bySmsCode(@FormParam("phone") String phone,
                            @FormParam("code") String code,
                            @FormParam("inviteCode") String inviteCode,
                            @FormParam("refreshToken") String refreshToken,
                            @FormParam("countryId") Integer countryId,
                            @FormParam("channelId") Long channelId) {

        if (countryId == null) {
            countryId = 1;
        }


        String areaCode = dataService.getAreaCode(countryId);


        // 国际手机号 拼上区号
        String fullPhone = "86".equals(areaCode) ? phone : areaCode + phone;

        // 判断手机号错误次数
        if (smsService.getPhoneInvalidTimes(fullPhone) > 2) {
            log.warn("验证码错误次数上限");
            return new Result(GlobalApiError.OperationFailed, "验证码尝试错误次数过多，请稍后再试");
        }

        // 手机号验证码校验
        if (!smsService.checkSmsCode(SmsActionEnum.LOGIN, fullPhone, code)) {
            return new Result(ApiError.LoginFailed, "验证码无效或已过期，请重新输入");
        }

        if (!StringUtil.isNullOrEmpty(refreshToken)) {
            return byRefreshToken(phone, refreshToken);
        }

        // 删除验证码
        smsService.delSmsCode(SmsActionEnum.LOGIN, fullPhone);

        if (channelId == null) {
            channelId = this.getHeader().getChannelId();
        }
        return handleLoginByPhone(phone, inviteCode, RegisterTypeEnum.SMS.getValue(), countryId, areaCode, channelId);
    }

    /**
     * h5短信验证登录
     *
     * @param phone
     * @param code
     * @param countryId
     * @return
     */
    @POST
    @Path("h5BySmsCodeLogin")
    @PermitGuest
    @Override
    public Result h5BySmsCodeLogin(@FormParam("phone") String phone,
                                   @FormParam("code") String code,
                                   @FormParam("imageCode") String imageCode,
                                   @FormParam("imagecodeMd5") String imagecodeMd5,
                                   @FormParam("countryId") Integer countryId) {
        if (countryId == null) {
            countryId = 1;
        }

        //验证图形验证码
        if (StringUtils.isEmpty(imageCode) || StringUtils.isEmpty(imagecodeMd5) || !imagecodeMd5.equalsIgnoreCase(SafeUtil.getAlphabetVcodeMd5(imageCode))) {
            return new Result(ApiError.LoginFailed, "图形验证码错误，请重新输入");
        }

        String areaCode = dataService.getAreaCode(countryId);

        // 国际手机号 拼上区号
        String fullPhone = "86".equals(areaCode) ? phone : areaCode + phone;

        // 判断手机号错误次数
        if (smsService.getPhoneInvalidTimes(fullPhone) > 2) {
            log.warn("验证码错误次数上限");
            return new Result(GlobalApiError.OperationFailed, "验证码尝试错误次数过多，请稍后再试");
        }

        // 手机号验证码校验
        if (!smsService.checkSmsCode(SmsActionEnum.LOGIN, fullPhone, code)) {
            return new Result(ApiError.LoginFailed, "验证码无效或已过期，请重新输入");
        }

        // 删除验证码
        smsService.delSmsCode(SmsActionEnum.LOGIN, fullPhone);


        Long userId = securityService.getUserIdByPhone(phone, areaCode);
        if (userId == null || userId <= 0) {
            return new Result(ApiError.LoginFailed, "用户不存在");
        }

        UserDO userDO = userService.getUserById(userId);
        MutableResult mResult = new MutableResult();
        mResult.put("sign", SafeUtil.getSign(userDO.getUserId().toString()));
        mResult.put("userId", userDO.getUserId());
        mResult.put("isAuth", userService.isAuthExist(userId));
        mResult.put("userAvatar", userDO.getUserAvatar());
        mResult.put("userNo", userDO.getUserNo());
        mResult.put("userName", userDO.getUserName());

        return mResult.build();
    }

    /**
     * 获取图片验证码
     * @return
     */
    @POST
    @Path("getImageVerifyCode")
    @PermitGuest
    @Override
    public Result getImageVerifyCode() {
        String vcode = SafeUtil.getAlphabetVcode();
        // 验证码md5
        String vcodeMd5 = SafeUtil.getAlphabetVcodeMd5(vcode);
        // 验证码图片
        BufferedImage vcodeImage = ImageUtil.getValidationImage(vcode);

        // 图片转byte数组
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        String imageString = "";
        try {
            ImageIO.write(vcodeImage, "jpg", baos);
            baos.flush();
            byte[] imageInByte = baos.toByteArray();
            // 图片base64
            imageString = Base64Utils.encodeToString(imageInByte);
            baos.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        MutableResult mResult = new MutableResult();
        mResult.put("image", imageString);
        mResult.put("imagecodeMd5", vcodeMd5);

        return mResult.build();
    }

    /**
     * 发送短信验证码
     *
     * @param phone
     * @param codeType
     * @param isDevSend
     * @param countryId
     * @param sessionId
     * @param sig
     * @param token
     * @param scene
     * @return
     */
    @POST
    @Path("sendSmsCode")
    @PermitGuest
    @Override
    public Result sendSmsCode(@FormParam("phone") String phone,
                              @FormParam("codeType") int codeType,
                              @FormParam("isDevSend") boolean isDevSend,
                              @FormParam("countryId") Integer countryId,
                              @FormParam("sessionId") String sessionId,
                              @FormParam("sig") String sig,
                              @FormParam("token") String token,
                              @FormParam("scene") String scene) {
        String deviceId = this.getHeader().getDeviceId();

        Long channelId = this.getHeader().getChannelId();
        // 开启人机校验
        if (channelService.isMachineValidateEnable(channelId)) {
            if (StringUtils.isEmpty(sessionId) || StringUtils.isEmpty(sig) || StringUtils.isEmpty(token) || StringUtils.isEmpty(scene)) {
                log.info("需要人机校验");
                MutableResult mResult = new MutableResult();
                mResult.put("needValid", true);
                return mResult.build();
            } else if (!"ic_message_h5".equals(scene)) {
                log.info("人机签名不合法");
                return new Result(GlobalApiError.ServerError, "人机签名不合法");
            } else {
                if (!smsService.machineValidate(channelId, sessionId, sig, token, scene, this.getIpAddress())) {
                    return new Result(GlobalApiError.ServerError, "人机校验失败");
                }
            }
        }

        if (!isDev()) {
            // 判断设备有效性
            if (ClientType.iOS == this.getHeader().getClientType() || ClientType.Android == this.getHeader().getClientType()) {
                if (!deviceService.isExists(deviceId)) {
                    log.info("绕过设备注册，已过滤");
                    MutableResult mResult = new MutableResult();
                    mResult.put("code", "");
                    mResult.put("isNew", true);
                    return mResult.build();
                }

                if (deviceService.isLocked(deviceId)) {
                    return new Result(GlobalApiError.OperationFailed, "您的设备已被封禁，请联系客服");
                }

                if (statPlatformService.getSmsDeviceCountToday(deviceId) >= 20) {
                    log.info("deviceId 触发流控");
                    return new Result(GlobalApiError.OperationFailed, "触发流控，请联系客服");
                }
            }

            String ip = this.getIpAddress();
            int ipLimit = 10000;
            if (statPlatformService.getSmsIpCountToday(ip) >= ipLimit) {
                log.info("ip 触发流控 {}", ip);
                return new Result(GlobalApiError.OperationFailed, "触发流控，请联系客服");
            }
        }

        if (countryId == null) {
            countryId = 1;
        }

        SmsActionEnum actionEnum = SmsActionEnum.toEnum(codeType);
        if (actionEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "短信验证码类型错误");
        }

        String areaCode = dataService.getAreaCode(countryId);
        // 国际手机号 拼上区号
        String fullPhone = "86".equals(areaCode) ? phone : areaCode + phone;

        boolean phoneEx = securityService.isPhoneExists(phone, areaCode);

        // 如果是绑定的话
        if (actionEnum.getCode() == SmsActionEnum.BIND.getCode()) {
            if (phoneEx) {
                return new Result(GlobalApiError.OperationFailed, "该手机号已经绑定");
            }
        }

        SendResult sendResult = null;

//        if (zhimaChannel.equals(this.getHeader().getChannelId())) {
//            log.info("该渠道需要语音验证码");
//            sendResult = smsService.sendVoiceCode(actionEnum, fullPhone, isDevSend, this.getHeader().getChannelId(), areaCode, this.getIpAddress(), deviceId);
//        } else {
        sendResult = smsService.sendSmsCode(actionEnum, fullPhone, isDevSend, this.getHeader().getChannelId(), areaCode, this.getIpAddress(), deviceId);
//        }

        if (!sendResult.isSuccess()) {
            return new Result(GlobalApiError.OperationFailed, "发送失败," + sendResult.getMsg());
        }

        MutableResult mResult = new MutableResult();
        mResult.put("code", Optional.ofNullable(sendResult.getData()).map(Object::toString).orElse(""));

        // 新用户判断
        mResult.put("isNew", !phoneEx);

        return mResult.build();
    }

    /**
     * 测试环境发送短信
     *
     * @param phone
     * @param codeType
     * @param countryId
     * @param sessionId
     * @param sig
     * @param token
     * @param scene
     * @return
     */
    @POST
    @Path("sendSmsCodeOnlyTest")
    @PermitGuest
    public Result sendSmsCodeOnlyTest(@FormParam("phone") String phone,
                                      @FormParam("codeType") int codeType,
                                      @FormParam("countryId") Integer countryId,
                                      @FormParam("sessionId") String sessionId,
                                      @FormParam("sig") String sig,
                                      @FormParam("token") String token,
                                      @FormParam("scene") String scene) {
        if (!isDev()) {
            return Result.ofBadRequest("only test");
        }
        String deviceId = this.getHeader().getDeviceId();

        if (countryId == null) {
            countryId = 1;
        }

        SmsActionEnum actionEnum = SmsActionEnum.toEnum(codeType);
        if (actionEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "短信验证码类型错误");
        }

        String areaCode = dataService.getAreaCode(countryId);
        // 国际手机号 拼上区号
        String fullPhone = "86".equals(areaCode) ? phone : areaCode + phone;
        boolean phoneEx = securityService.isPhoneExists(phone, areaCode);

        // 如果是绑定的话
        if (actionEnum.getCode() == SmsActionEnum.BIND.getCode()) {
            if (phoneEx) {
                return new Result(GlobalApiError.OperationFailed, "该手机号已经绑定");
            }
        }

        String verifyCode = smsService.sendSmsCodeOnlyTest(actionEnum, fullPhone, this.getHeader().getChannelId(), areaCode, this.getIpAddress(), deviceId);
        MutableResult mResult = new MutableResult();
        mResult.put("code", verifyCode);

        // 新用户判断
        mResult.put("isNew", !phoneEx);

        return mResult.build();
    }

    /**
     * h5发送短信验证码
     *
     */
    @POST
    @Path("sendH5SmsCode")
    @PermitGuest
    @Override
    public Result sendH5SmsCode(@FormParam("phone") String phone,
                                @FormParam("codeType") int codeType,
                                @FormParam("countryId") Integer countryId,
                                @FormParam("imageCode") String imageCode,
                                @FormParam("imagecodeMd5") String imagecodeMd5,
                                @FormParam("isDevSend") boolean isDevSend
    ) {
        //验证图形验证码
        if (StringUtils.isEmpty(imageCode) || StringUtils.isEmpty(imagecodeMd5) || !imagecodeMd5.equalsIgnoreCase(SafeUtil.getAlphabetVcodeMd5(imageCode))) {
            return new Result(GlobalApiError.OperationFailed, "图形验证码错误，请重新输入");
        }
        if (countryId == null) {
            countryId = 1;
        }

        SmsActionEnum actionEnum = SmsActionEnum.toEnum(codeType);
        if (actionEnum == null) {
            return new Result(GlobalApiError.OperationFailed, "短信验证码类型错误");
        }

        String areaCode = dataService.getAreaCode(countryId);
        // 国际手机号 拼上区号
        String fullPhone = "86".equals(areaCode) ? phone : areaCode + phone;

        boolean phoneEx = securityService.isPhoneExists(phone, areaCode);

        // 如果是绑定的话
        if (actionEnum.getCode() == SmsActionEnum.BIND.getCode()) {
            if (phoneEx) {
                return new Result(GlobalApiError.OperationFailed, "该手机号已经绑定");
            }
        }

        SendResult sendResult = null;


        sendResult = smsService.sendSmsCode(actionEnum, fullPhone, isDevSend, this.getHeader().getChannelId(), areaCode, this.getIpAddress(), "");

        if (!sendResult.isSuccess()) {
            return new Result(GlobalApiError.OperationFailed, "发送失败," + sendResult.getMsg());
        }

        MutableResult mResult = new MutableResult();
        mResult.put("code", Optional.ofNullable(sendResult.getData()).map(Object::toString).orElse(""));

        // 新用户判断
        mResult.put("isNew", !phoneEx);

        return mResult.build();

    }

    /**
     * 手机号一键登录
     *
     * @param cert
     * @param inviteCode
     * @return
     */
    @POST
    @Path("byOneKey")
    @PermitGuest
    @Override
    public Result byOneKey(@FormParam("cert") String cert,
                           @FormParam("inviteCode") String inviteCode,
                           @FormParam("refreshToken") String refreshToken) {
        String phone = thirdAuthService.getOneKeyPhoneByCert(this.getHeader().getChannelId(), cert, getHeader().getClientType().getValue());

        if (StringUtils.isBlank(phone)) {
            return new Result(GlobalApiError.OperationFailed, "错误的授权码");
        }

        if (!StringUtil.isNullOrEmpty(refreshToken)) {
            return byRefreshToken(phone, refreshToken);
        }

        return handleLoginByPhone(phone, inviteCode, RegisterTypeEnum.ONE_KEY.getValue(), 1, "86", 0L);
    }

    /**
     * 第三方 OAuth 登录
     *
     * @param type
     * @param code
     * @param inviteCode
     * @return
     */
    @POST
    @Path("byThirdPartyOAuth")
    @PermitGuest
    @Override
    public Result byThirdPartyOAuth(@FormParam("type") int type,
                                    @FormParam("code") String code,
                                    @FormParam("inviteCode") String inviteCode) {
        LoginTypeEnum loginType = LoginTypeEnum.toEnum(type);
        if (loginType == null || type < LoginTypeEnum.ByWxApp.getValue()) {
            return new Result(ApiError.LoginFailed, "不支持该第三方登录");
        }
        Long channelId = Optional.ofNullable(this.getHeader().getChannelId()).orElse(0L);
        // 第三方登录处理
        switch (loginType) {
            case ByWxApp:
                return loginByWxApp(channelId, loginType, code);
            case ByQQApp:
                return loginByQQApp(channelId, loginType, code);
            case ByWbApp:
                return loginByWbApp(channelId, loginType, code);
            case ByHuawei:
                return loginByHuawei(channelId, loginType, code);
            case ByFacebook:
                return loginByFacebook(channelId, loginType, code);
            case ByGoogle:
                return loginByGoogle(channelId, loginType, code);
            default:
                log.info("暂未开通此登录方式", loginType.getName());
                break;
        }

        return new Result(ApiError.LoginFailed);
    }

    /**
     * 校验短信验证码
     *
     * @param codeType
     * @param phone
     * @param smsCode
     * @param countryId
     * @return
     */
    @POST
    @Path("verifySmsCode")
    @PermitGuest
    @Override
    public Result verifySmsCode(@FormParam("codeType") int codeType,
                                @FormParam("phone") String phone,
                                @FormParam("smsCode") String smsCode,
                                @FormParam("countryId") Integer countryId) {
        SmsActionEnum smsActionEnum = SmsActionEnum.toEnum(codeType);

        if (smsActionEnum == null) {
            return new Result(GlobalApiError.ServerError, "无效的短信类型");
        }

        if (countryId == null) {
            countryId = 1;
        }
        String areaCode = dataService.getAreaCode(countryId);

        // 国际手机号 拼上区号
        String fullPhone = "86".equals(areaCode) ? phone : areaCode + phone;
        if (smsService.checkSmsCode(smsActionEnum, fullPhone, smsCode)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError, "验证码错误");
    }

    /**
     * 密码登录
     *
     * @param phone
     * @param password
     * @param countryId
     * @return
     */
    @POST
    @Path("byPassword")
    @PermitGuest
    @Override
    public Result byPassword(@FormParam("phone") String phone,
                             @FormParam("password") String password,
                             @FormParam("countryId") Integer countryId) {
        if (countryId == null) {
            countryId = 1;
        }

        String areaCode = dataService.getAreaCode(countryId);

        // 判断是否注册
        Long userId = securityService.getUserIdByPhone(phone, areaCode);
        if (userId != null) {
            // 判断密码错误次数是否超限
            if (securityService.isPassErrorOverTimes(userId)) {
                return new Result(GlobalApiError.OperationFailed, "您短时间内密码错误次数过多，请稍后再试");
            }

            // 判断密码是否设置
            if (securityService.isUserPassSet(userId)) {
                // 判断密码是否正确
                if (securityService.verifyPass(userId, password)) {
                    // 登录成功
                    return loginSuccess(LoginTypeEnum.ByPhoneAndPassword, userId, false, false);
                } else {
                    return new Result(GlobalApiError.OperationFailed, "账号或密码错误");
                }
            }
        }

        return new Result(GlobalApiError.OperationFailed, "请使用验证码登录");
    }

    /**
     * 激活
     *
     * @return
     */
    @POST
    @Path("activation")
    @PermitGuest
    @Override
    public Result activation() {
        RiskParms parms = risk(null);
        parms.setAdvertisingId(this.getHeaderString("idfa"));
        parms.setApputm(this.getHeader().getChannelId().toString());
        riskService.activationAudit(parms);

        return new Result();
    }


    private RiskParms risk(Long userId) {
        RiskParms p = new RiskParms();

        p.setUserID(userId);
        String phone = this.getRequest().getParameter("phone");
        if (StringUtils.isNotEmpty(phone)) {
            p.setPhone(SecureUtil.md5(phone));
            p.setCountryCode("0086");
        }
        p.setIp(this.getIpAddress());
        p.setTimestamp(System.currentTimeMillis());
        p.setDeviceId(this.getHeaderString("rdid"));
        p.setOs(this.getHeader().getClientType().getName());
        p.setAppVersion(this.getHeader().getAppVersion());
        if (StringUtils.isNotEmpty(this.getHeader().getLongitude())) {
            p.setLng(Double.parseDouble(this.getHeader().getLongitude()));
        }
        if (StringUtils.isNotEmpty(this.getHeader().getLatitude())) {
            p.setLat(Double.parseDouble(this.getHeader().getLatitude()));
        }
        log.info("risk:" + JSONObject.toJSONString(p));
        return p;
    }

    /**
     * 设置密码
     *
     * @param password
     * @return
     */
    @POST
    @Path("setPassword")
    @Override
    public Result setPassword(@FormParam("password") String password) {
        Long userId = this.getToken().getUserId();
        // 判断是否绑定手机号
        if (!securityService.isPhoneBind(userId)) {
            return new Result(GlobalApiError.ServerError, "请先绑定手机号");
        }
        if (securityService.setUserPass(userId, password)) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 根据token获取个人资料
     *
     * @param token
     * @return
     */
    @GET
    @Path("getProfileByToken")
    @PermitAll
    @Override
    public Result getProfileByToken(@QueryParam("token") String token) {
        TokenBO tokenBO = securityService.getToken(token);
        if (tokenBO != null) {
            UserProfile profile = userService.getProfile(tokenBO.getUserId());
            if (profile != null) {
                AES aes = new AES(API_AES_SECRET.getBytes());
                String encryptProfile = aes.encryptBase64(JSON.toJSONString(profile));
                return new Result(encryptProfile);
            } else {
                return new Result(GlobalApiError.ServerError, "未获取到用户信息");
            }
        }
        return new Result(GlobalApiError.ServerError, "token无效");
    }

    /**
     * 获取 RTM 令牌
     *
     * @return
     */
    @GET
    @Path("getRtmToken")
    @Override
    public Result getRtmToken() {
        Long userId = this.getToken().getUserId();
        redisService.set(RedisUserKeys.getUserAppVer(userId), getHeader().getAppVersion(), 30, TimeUnit.DAYS);
        String rtmToken = imService.getRongCloudToken(userId);
        return new Result(rtmToken);
    }

    /**
     * 微信登录
     *
     * @param channelId
     * @param loginType
     * @param code
     * @return
     */
    private Result loginByWxApp(Long channelId, LoginTypeEnum loginType, String code) {
        WxAccessTokenResp wxAccessToken = thirdAuthService.getWxAppAccessToken(channelId, code);
        if (wxAccessToken == null) {
            return new Result(ApiError.LoginFailed, "服务器出了点小意外，未获取到微信accessToken");
        }

        WxAppUserInfoResp infoResp = thirdAuthService.getWxAppUserInfo(channelId, wxAccessToken.getAccessToken());
        if (infoResp == null) {
            return new Result(ApiError.LoginFailed, "服务器出了点小意外，未获取到微信用户信息");
        }

        Long userId = securityService.getUserIdByUnionId(infoResp.getUnionId(), channelId, loginType.getValue());
        boolean isNew = false;
        if (userId == null) {
            UserAddDTO userAddDTO = UserAddDTO.builder()
                    .channelId(channelId)
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .registerType(RegisterTypeEnum.WX.getValue())
                    .nickName(infoResp.getNickname())
                    .sex(infoResp.getSex())
                    .clientIp(this.getIpAddress())
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .countryId(1)
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .headImgUrl(infoResp.getHeadImgUrl()).build();
            userId = userService.addUser(userAddDTO);
            if (userId == null) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败");
            }

            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }

            isNew = true;

            securityService.saveOauthInfo(userId, channelId, OAuthTypeEnum.WEIXIN_APP.getType(), infoResp.getUnionId(), infoResp.getOpenId());

        }
        return loginSuccess(loginType, userId, isNew, false);
    }

    /**
     * qq登录
     *
     * @param channelId
     * @param loginType
     * @param code
     * @return
     */
    private Result loginByQQApp(Long channelId, LoginTypeEnum loginType, String code) {

        String qqUnionId = thirdAuthService.getQqUnionId(channelId, code);
        if (StringUtils.isEmpty(qqUnionId)) {
            return new Result(ApiError.LoginFailed, "服务器出了点小意外，未获取到 QQ unionId");
        }

        Long userId = securityService.getUserIdByUnionId(qqUnionId, channelId, loginType.getValue());
        boolean isNew = false;
        if (userId == null) {
            String qqOpenId = thirdAuthService.getQqOpenId(channelId, code);
            if (StringUtils.isEmpty(qqOpenId)) {
                return new Result(ApiError.LoginFailed, "服务器出了点小意外，未获取到 QQ qqOpenId");
            }

            QqUserInfoResp userInfo = thirdAuthService.getQqUserInfo(channelId, code, qqOpenId);
            if (userInfo == null) {
                return new Result(ApiError.LoginFailed, "服务器出了点小意外，未获取到 QQ 用户信息");
            }
            UserAddDTO userAddDTO = UserAddDTO.builder()
                    .channelId(channelId)
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .registerType(RegisterTypeEnum.QQ.getValue())
                    .nickName(userInfo.getNickname())
                    .sex("男".equals(userInfo.getGender()) ? SexEnum.MALE.getCode() : SexEnum.FEMALE.getCode())
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .countryId(1)
                    .clientIp(this.getIpAddress())
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .headImgUrl(userInfo.getFigureUrlQq1()).build();
            userId = userService.addUser(userAddDTO);
            if (userId == null) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败");
            }

            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }
            isNew = true;

            securityService.saveOauthInfo(userId, channelId, OAuthTypeEnum.QQ_APP.getType(), qqUnionId, qqOpenId);
        }

        return loginSuccess(loginType, userId, isNew, false);
    }

    /**
     * 微博登录
     *
     * @param channelId
     * @param loginType
     * @param code
     * @return
     */
    private Result loginByWbApp(Long channelId, LoginTypeEnum loginType, String code) {
        WbUserInfoResp wbUserInfo = thirdAuthService.getWbUserInfo(channelId, code);
        if (wbUserInfo == null) {
            return new Result(ApiError.LoginFailed, "服务器出了点小意外，未获取到微博用户信息");
        }
        Long userId = securityService.getUserIdByUnionId(wbUserInfo.getId().toString(), channelId, loginType.getValue());
        boolean isNew = false;
        if (userId == null) {
            Integer sex = null;
            if (!StringUtil.isNullOrEmpty(wbUserInfo.getGender())) {
                switch (wbUserInfo.getGender()) {
                    case "m":
                        sex = SexEnum.MALE.getCode();
                        break;
                    case "f":
                        sex = SexEnum.FEMALE.getCode();
                        break;
                    default:
                        sex = SexEnum.UNKNOWN.getCode();
                        break;
                }
            }
            UserAddDTO userAddDTO = UserAddDTO.builder()
                    .channelId(channelId)
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .registerType(RegisterTypeEnum.WB.getValue())
                    .nickName(wbUserInfo.getName())
                    .sex(sex)
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .countryId(1)
                    .clientIp(this.getIpAddress())
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .headImgUrl(wbUserInfo.getProfileImageUrl()).build();
            userId = userService.addUser(userAddDTO);
            if (userId == null) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败");
            }

            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }
            isNew = true;

            securityService.saveOauthInfo(userId, channelId, OAuthTypeEnum.WEIBO_APP.getType(), wbUserInfo.getId().toString(), "");
        }

        return loginSuccess(loginType, userId, isNew, false);
    }

    /**
     * 华为登录
     *
     * @param channelId
     * @param loginType
     * @param code
     * @return
     */
    private Result loginByHuawei(Long channelId, LoginTypeEnum loginType, String code) {
        HuaweiAccessTokenResp accessTokenResp = thirdAuthService.getHuaweiAccessToken(channelId, code);
        if (accessTokenResp == null || StringUtils.isBlank(accessTokenResp.getAccessToken())) {
            return new Result(ApiError.LoginFailed, "华为授权登录失败，请尝试其他方式登录");
        }

        HuaweiTokenInfoResp tokenInfoResp = thirdAuthService.getHuaweiTokenInfo(channelId, accessTokenResp.getAccessToken());
        if (tokenInfoResp == null) {
            return new Result(ApiError.LoginFailed, "华为授权登录失败，请尝试其他方式登录");
        }

        Long userId = securityService.getUserIdByUnionId(tokenInfoResp.getUnionId(), channelId, loginType.getValue());
        boolean isNew = false;
        if (userId == null) {
            UserAddDTO userAddDTO = UserAddDTO.builder()
                    .channelId(channelId)
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .countryId(1)
                    .clientIp(this.getIpAddress())
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .registerType(RegisterTypeEnum.HUAWEI.getValue()).build();
            userId = userService.addUser(userAddDTO);
            if (userId == null) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败");
            }

            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }
            isNew = true;

            securityService.saveOauthInfo(userId, channelId, OAuthTypeEnum.HUAWEI_APP.getType(), tokenInfoResp.getUnionId(), tokenInfoResp.getOpenId());
        }

        return loginSuccess(loginType, userId, isNew, false);
    }

    /**
     * facebook登录
     *
     * @param channelId
     * @param loginType
     * @param code
     * @return
     */
    private Result loginByFacebook(Long channelId, LoginTypeEnum loginType, String code) {

        FacebookUserInfoResp userInfo = thirdAuthService.getFacebookUserInfo(channelId, code);
        if (userInfo == null) {
            return new Result(ApiError.LoginFailed, "华为授权登录失败，请尝试其他方式登录");
        }

        Long userId = securityService.getUserIdByUnionId(userInfo.getClientId(), channelId, loginType.getValue());
        boolean isNew = false;
        if (userId == null) {
            UserAddDTO userAddDTO = UserAddDTO.builder()
                    .channelId(channelId)
                    .nickName(userInfo.getName())
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .clientIp(this.getIpAddress())
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .registerType(RegisterTypeEnum.FACEBOOK.getValue()).build();
            userId = userService.addUser(userAddDTO);
            if (userId == null) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败");
            }

            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }
            isNew = true;

            securityService.saveOauthInfo(userId, channelId, OAuthTypeEnum.HUAWEI_APP.getType(), userInfo.getClientId(), userInfo.getId());
        }

        return loginSuccess(loginType, userId, isNew, false);
    }

    /**
     * google登录
     *
     * @param channelId
     * @param loginType
     * @param code
     * @return
     */
    private Result loginByGoogle(Long channelId, LoginTypeEnum loginType, String code) {

        GoogleUserInfoResp userInfo = thirdAuthService.getGoogleUserInfo(channelId, code);
        if (userInfo == null) {
            return new Result(ApiError.LoginFailed, "华为授权登录失败，请尝试其他方式登录");
        }

        Long userId = securityService.getUserIdByUnionId(userInfo.getClientId(), channelId, loginType.getValue());
        boolean isNew = false;
        if (userId == null) {
            UserAddDTO userAddDTO = UserAddDTO.builder()
                    .channelId(channelId)
                    .nickName(userInfo.getName())
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .clientIp(this.getIpAddress())
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .registerType(RegisterTypeEnum.GOOGLE.getValue()).build();
            userId = userService.addUser(userAddDTO);
            if (userId == null) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败");
            }

            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }

            isNew = true;

            securityService.saveOauthInfo(userId, channelId, OAuthTypeEnum.HUAWEI_APP.getType(), userInfo.getClientId(), userInfo.getId());
        }

        return loginSuccess(loginType, userId, isNew, false);
    }

    /**
     * 处理手机号登录逻辑
     *
     * @param phone
     * @param inviteCode
     * @param registerType
     * @param countryId
     * @param areaCode
     * @return
     */
    private Result handleLoginByPhone(String phone, String inviteCode, Integer registerType, Integer countryId, String areaCode, Long channelId) {
        // 注册判断
        Long userId = securityService.getUserIdByPhone(phone, areaCode);
        boolean isNew = false;
        boolean needProfile = false;

        // 为新用户时
        if (userId == null) {
            isNew = true;
            needProfile = true;
            if (channelId == null || channelId == 0) {
                channelId = this.getHeader().getChannelId();
            }
            // 创建用户
            UserAddDTO userAddDTO = UserAddDTO.builder()
                    .channelId(channelId)
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .registerType(registerType)
                    .userPhone(phone)
                    .countryId(countryId)
                    .areaCode(areaCode)
                    .clientIp(this.getIpAddress())
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .userIsMarketAuditor(BoolIntVal.False)
                    .userIsRobot(BoolIntVal.False)
                    .clientIp(this.getIpAddress())
                    .build();

            // 第三方用户信息
            ThirdPartyUser user3Party = thirdPartyService.getUserByPhone(channelId, areaCode, phone);
            if (user3Party != null) {
                BeanUtils.copyProperties(user3Party, userAddDTO);
                needProfile = false;
                log.info("使用第三方用户信息登陆 phone={} user3Party={}", phone, user3Party);
            }

            // 是否必填邀请码
            Integer inviteCodeEnable = channelService.getInviteCodeEnable(this.getHeader().getChannelId());

            if (inviteCodeEnable != null && inviteCodeEnable == BoolIntVal.True) {
                if (StringUtils.isEmpty(inviteCode)) {
                    return new Result(GlobalApiError.ServerError, "请填写邀请码");
                }
            }

            // 邀请码判断
            if (StringUtils.isNotBlank(inviteCode)) {
                inviteCode = inviteCode.replaceAll("[\\$\\#]", "");
                Long inviterUserId = userService.getIdByInviteCode(inviteCode);
                if (inviterUserId == null) {
                    return new Result(GlobalApiError.OperationFailed, "邀请码无效");
                }
                userAddDTO.setInviterUserId(inviterUserId);
            }

            // 风控注册判断，通过就注册用户
            userId = userService.addUser(userAddDTO);
            if (userId == null) {
                return new Result(GlobalApiError.ServerError);
            }

            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }

        }

        // 风控登录判断，通过就登录用户
        return loginSuccess(LoginTypeEnum.ByPhoneAndSmsCode, userId, isNew, needProfile);
    }


    /**
     * 根据refreshToken登录
     *
     * @param phone
     * @param refreshToken
     * @return
     */
    private Result byRefreshToken(String phone, String refreshToken) {
        // 解密
        String token = SecureUtil.aes(AES_SECRET.getBytes()).decryptStr(refreshToken);
        String[] split = token.split(":");
        if (split.length != 2) {
            log.info("refreshToken解密后长度不合法");
            return new Result(ApiError.LoginFailed, "绑定失败，请重试");
        }
        Long channelId = Optional.ofNullable(this.getHeader().getChannelId()).orElse(0L);
        Long tempId = Long.valueOf(split[1]);
        String unionId = null;
        String openId = null;
        Long userId;
        String nickName = null;
        String headImgUrl = null;
        Integer sex = null;

        LoginTypeEnum loginType = LoginTypeEnum.toEnum(Convert.toInt(split[0]));
        RegisterTypeEnum registerType = null;
        switch (loginType) {
            case ByWxApp:
                registerType = RegisterTypeEnum.WX;
                WxAppUserInfoResp resp = JSON.parseObject(cacheService.get(RedisCacheKeys.getRefreshToken(tempId)), WxAppUserInfoResp.class);
                if (resp == null) {
                    log.info("refreshToken已过期");
                    return new Result(ApiError.LoginFailed, "绑定失败，请重试");
                }
                unionId = resp.getUnionId();
                openId = resp.getOpenId();
                nickName = resp.getNickname();
                sex = resp.getSex();
                headImgUrl = resp.getHeadImgUrl();
                break;
            case ByQQApp:
                registerType = RegisterTypeEnum.QQ;
                JSONObject jsonObject = JSON.parseObject(cacheService.get(RedisCacheKeys.getRefreshToken(tempId)));
                if (jsonObject == null) {
                    log.info("refreshToken已过期");
                    return new Result(ApiError.LoginFailed, "绑定失败，请重试");
                }
                QqUserInfoResp qqUserInfoResp = JSON.parseObject(jsonObject.getString("qqUserInfo"), QqUserInfoResp.class);
                unionId = jsonObject.getString("qqUnionId");
                openId = jsonObject.getString("qqOpenId");
                nickName = qqUserInfoResp.getNickname();
                sex = "男".equals(qqUserInfoResp.getGender()) ? SexEnum.MALE.getCode() : SexEnum.FEMALE.getCode();
                headImgUrl = qqUserInfoResp.getFigureUrlQq1();
                break;
            case ByWbApp:
                registerType = RegisterTypeEnum.WB;
                WbUserInfoResp wbUserInfoResp = JSON.parseObject(cacheService.get(RedisCacheKeys.getRefreshToken(tempId)), WbUserInfoResp.class);
                if (wbUserInfoResp == null) {
                    log.info("refreshToken已过期");
                    return new Result(ApiError.LoginFailed, "绑定失败，请重试");
                }
                unionId = wbUserInfoResp.getId().toString();
                nickName = wbUserInfoResp.getName();
                headImgUrl = wbUserInfoResp.getProfileImageUrl();
                if (!StringUtil.isNullOrEmpty(wbUserInfoResp.getGender())) {
                    switch (wbUserInfoResp.getGender()) {
                        case "m":
                            sex = SexEnum.MALE.getCode();
                            break;
                        case "f":
                            sex = SexEnum.FEMALE.getCode();
                            break;
                        default:
                            sex = SexEnum.UNKNOWN.getCode();
                            break;
                    }
                }
                break;
            case ByHuawei:
                registerType = RegisterTypeEnum.HUAWEI;
                HuaweiTokenInfoResp tokenInfoResp = JSON.parseObject(cacheService.get(RedisCacheKeys.getRefreshToken(tempId)), HuaweiTokenInfoResp.class);
                if (tokenInfoResp == null) {
                    return new Result(ApiError.LoginFailed, "绑定失败，请重试");
                }
                unionId = tokenInfoResp.getUnionId();
                nickName = "我是一颗小豆芽";
                sex = SexEnum.UNKNOWN.getCode();
                break;
            default:
                return new Result(ApiError.LoginFailed, "暂未开通此登录方式");
        }

        // 查询手机号是否已注册
        userId = securityService.getUserIdByPhone(phone, "");
        if (userId == null) {
            // 未注册的手机号 直接注册
            userId = userService.addUser(UserAddDTO.builder()
                    .channelId(channelId)
                    .deviceId(this.getHeader().getDeviceId())
                    .clientType(this.getHeader().getClientType())
                    .registerType(registerType.getValue())
                    .userPhone(phone)
                    .nickName(nickName)
                    .sex(sex)
                    .clientIp(this.getIpAddress())
                    .idfa(this.getHeaderString("idfa"))
                    .imeiMd5(this.getHeaderString("imei"))
                    .appVersion(this.getHeader().getAppVersion())
                    .riskDeviceId(this.getHeaderString("rdid"))
                    .countryId(1)
                    .headImgUrl(headImgUrl).build());
            if (userId == null) {
                log.info("registReault-注册失败设备已被风控：" + phone);
                return new Result(GlobalApiError.OperationFailed, "创建用户失败");
            }
            if (userId == -1) {
                return new Result(GlobalApiError.OperationFailed, "创建用户失败,设备已被风控");
            }

        } else {
            // 判断是否已绑定
            if (securityService.isUserOauthBind(userId, channelId, loginType.getValue())) {
                return new Result(GlobalApiError.OperationFailed, "该手机号已绑定微信");
            }
        }

        // 执行绑定
        securityService.saveOauthInfo(userId, channelId, loginType.getValue(), unionId, openId);

        // 销毁refreshToken
        cacheService.delKey(RedisCacheKeys.getRefreshToken(tempId));

        // 去登录
        return loginSuccess(loginType, userId, false);
    }

    private Result loginSuccess(LoginTypeEnum loginType, long userId, boolean isNew, boolean needProfile) {

        //登录风控
        String rdid = this.getHeaderString("rdid");
        if (riskService.getConfig("login") && StringUtils.isNotEmpty(rdid)) {
            GreenAuditTrace greenAuditTrace = riskService.loginAudit(risk(userId));
            //请求不通过放过
            if (greenAuditTrace.getRobotAuditStatus() != GreenAuditStatusEnum.PASSED_ROBOT) {
                //请求异常放过
                if (greenAuditTrace.getRobotAuditStatus() != GreenAuditStatusEnum.BLOCK_EXCEPTION) {
                    //请求超时放过
                    if (greenAuditTrace.getRobotAuditStatus() != GreenAuditStatusEnum.BLOCK_TIMEOUT) {
                        //开关控制是否采用拒绝结果
                        if (riskService.getConfig("useLoginResult")) {
                            //被验证机构直接拒绝的
                            return new Result(GlobalApiError.OperationFailed, "当前设备已被风控");
                        }
                    }
                }
            }
        }

        if (deviceService.isLocked(this.getHeader().getDeviceId())) {
            return new Result(GlobalApiError.OperationFailed, "当前设备异常或已被限制登录");
        }

        if (Optional.ofNullable(userService.getStatus(userId)).orElse(CommonStatus.Normal) != CommonStatus.Normal) {
            // 需要查询用户异常原因
            UserForbiddenDO forbiddenDO = userService.getForbiddenHis(userId);
            if (forbiddenDO != null) {
                return new Result(GlobalApiError.OperationFailed, String.format("封禁原因 %s ,解封时间 %s", forbiddenDO.getRemark(), getForbiddenEndTime(forbiddenDO)));
            }
            return new Result(GlobalApiError.OperationFailed, "此账户因异常行为，已被限制登录");
        }

        // 是否冷静期
        boolean isCoolPeriod = false;

        if (securityService.isCoolPeriod(userId)) {
            isCoolPeriod = true;
        } else if (securityService.isLogout(userId)) {
            return new Result(GlobalApiError.UserLogout);
        }

        // 创建登录记录
        UserLoginHisDTO hisDTO = new UserLoginHisDTO();
        hisDTO.setUserId(userId);
        hisDTO.setHisLoginType(loginType);
        hisDTO.setHisLoginResult(BoolIntVal.True);
        hisDTO.setHisLoginParams(JSON.toJSONString(this.getRequest().getParameterMap()));
        hisDTO.setHisHeaders(JSON.toJSONString(this.getHeader()));
        hisDTO.setHisIp(this.getIpAddress());
        Long hisId = securityService.addLoginHisAndSendMsg(hisDTO);

        // 创建令牌
        String token = securityService.createToken(this.getHeader().getClientType(), this.getHeader().getAppType(), userId, this.getHeader().getDeviceId(), 0, hisId, rdid);

        // 创建 IM 令牌
        String imToken = securityService.createImToken(userId, 0);
        if (!isNew) {
            // 触发登录事件
            UserLoginEventParam userLoginEventParam = new UserLoginEventParam();
            userLoginEventParam.setUserId(userId);
            userLoginEventParam.setChannelId(this.getHeader().getChannelId());
            userLoginEventParam.setLoginType(loginType.getValue());
            userLoginEventParam.setFirstLogin(isNew);
            triggerService.emitEvent(TriggerEventEnum.USER_LOGIN, userLoginEventParam);

            // 兼容腾讯云IM迁移融云时部分用户没有在融云注册所以没有被禁言
            int wealthLevel = userService.getWealthLevelByUserId(userId);
            long userCreateTime = Optional.ofNullable(userService.getCreateTime(userId)).orElse(0L);
            if (wealthLevel == 1 && userCreateTime < 1624204800000L) {
                // 融云禁言
                imService.setUserNoSpeaking(userId, Integer.MAX_VALUE, Integer.MAX_VALUE);
            }
        }


        MutableResult mResult = new MutableResult();
        mResult.put("userId", userId);
        mResult.put("token", token);
        mResult.put("imToken", imToken);
        mResult.put("rtmToken", imToken);
        mResult.put("isNew", isNew);
        mResult.put("needProfile", needProfile);
        mResult.put("isCoolPeriod", isCoolPeriod);
        mResult.put("coolPeriod", isCoolPeriod ? securityService.getLogoutTime(userId) : null);
        return mResult.build();
    }

    /**
     * 登录成功
     *
     * @param loginType
     * @param userId
     * @param isNew
     * @return
     */
    private Result loginSuccess(LoginTypeEnum loginType, long userId, boolean isNew) {
        return loginSuccess(loginType, userId, isNew, isNew);
    }


    /**
     * 根据用户ID获取用户是否实名
     *
     * @param userId
     * @return
     */
    @POST
    @Path("getUserIsAuth")
    @PermitGuest
    @Override
    public Result getUserIsAuth(@FormParam("userId") Long userId) {
        MutableResult mResult = new MutableResult();
        mResult.put("isAuth", userService.isAuthExist(userId));
        return mResult.build();
    }


    public String getForbiddenEndTime(UserForbiddenDO userForbiddenDO) {
        ForbiddenDurationTypeEnum durationTypeEnum = ForbiddenDurationTypeEnum.toEnum(userForbiddenDO.getDurationType());
        Long duration = 0L;
        switch (durationTypeEnum) {
            case ONE_HOUR:
                duration = TimeConstants.ONE_HOUR_MILLS;
                break;
            case THREE_HOUR:
                duration = TimeConstants.THREE_HOUR_MILLS;
                break;
            case SEVEN_HOUR:
                duration = (TimeConstants.SEVEN_HOUR_MILLS);
                break;
            case ONE_DAY:
                duration = (TimeConstants.ONE_DAY_MILLS);
                break;
            case THREE_DAY:
                duration = (TimeConstants.THREE_DAY_MILLS);
                break;
            case SEVEN_DAY:
                duration = (TimeConstants.SEVEN_DAY_MILLS);
                break;
            case FOURTEEN_DAY:
                duration = (TimeConstants.FOURTEEN_DAY_MILLS);
                break;
            case THIRTY_DAY:
                duration = (TimeConstants.THIRTY_DAY_MILLS);
                break;
            case TEN_YEARS:
                duration = (TimeConstants.TEN_YEAR_MILLS);
                break;
            case FOREVER:
                duration = Long.valueOf(DateUtil.DAY_OF_MILLION) * 999;
                break;
            default:
                break;
        }
        Long endTime = userForbiddenDO.getCreateTime() + duration;
        return DateUtil.formatDate(endTime, "yyyy.MM.dd");
    }

}
