package com.starbuds.server.api.common.v1.impl;

import com.starbuds.server.api.common.inf.PermitGuest;
import com.starbuds.server.api.common.v1.facade.SearchFacade;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.data.RtcRoomSearchReq;
import com.starbuds.server.common.pojo.data.UserSearchReq;
import com.starbuds.server.common.service.data.MusicSearchService;
import com.starbuds.server.common.service.data.RtcRoomSearchService;
import com.starbuds.server.common.service.data.UserSearchService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;

/**
 * 搜索接口实现
 *
 * @Author jiangxiaowei
 * @DateTime 2019/12/28 16:25
 */
@Path("/v1/search")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class SearchFacadeImpl extends BaseFacade implements SearchFacade {

    @Reference(version = "1.0.0")
    UserSearchService userSearchService;

    @Reference(version = "1.0.0")
    RtcRoomSearchService rtcRoomSearchService;

    @Reference(version = "1.0.0")
    MusicSearchService musicSearchService;

    /**
     * 搜索用户
     *
     * @param content  搜索内容
     * @param pageNum  当前页码
     * @param pageSize 页面大小
     * @return
     */
    @POST
    @Path("searchContent")
    @PermitGuest
    @Override
    public Result searchContent(@FormParam("content") String content,
                                @FormParam("pageNum") int pageNum,
                                @FormParam("pageSize") int pageSize) {
        UserSearchReq searchReq = new UserSearchReq();
        searchReq.setParams(content);
        searchReq.setPageNum(pageNum);
        searchReq.setPageSize(pageSize);
        searchReq.setCurUserId(this.getToken() == null ? null : this.getToken().getUserId());
        return new Result(userSearchService.searchUser(searchReq));
    }

    /**
     * 搜索语聊房间
     *
     * @param content   搜索内容
     * @param pageIndex 当前页码
     * @param pageSize  页面大小
     * @return
     */
    @POST
    @Path("searchRtcRoom")
    @PermitGuest
    @Override
    public Result searchRtcRoom(@FormParam("content") String content,
                                @FormParam("pageIndex") int pageIndex,
                                @FormParam("pageSize") int pageSize) {
        RtcRoomSearchReq searchReq = new RtcRoomSearchReq();
        searchReq.setParams(content);
        searchReq.setPageIndex(pageIndex);
        searchReq.setPageSize(pageSize);
        return new Result(rtcRoomSearchService.searchRtcRoom(searchReq));
    }

    /**
     * 搜索音乐
     *
     * @param content   搜索内容
     * @param pageIndex 当前页码
     * @param pageSize  页面大小
     * @return
     */
    @POST
    @Path("searchMusic")
    @Override
    public Result searchMusic(@FormParam("content") String content,
                              @FormParam("pageIndex") int pageIndex,
                              @FormParam("pageSize") int pageSize) {
        return new Result(musicSearchService.searchMusic(content, pageIndex, pageSize));
    }
}
