package com.starbuds.server.api.common.v1.impl;

import com.starbuds.server.api.common.inf.PermitGuest;
import com.starbuds.server.api.common.v1.facade.StatFacade;
import com.starbuds.server.common.constant.common.ClientType;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import java.math.BigDecimal;
import java.util.Optional;

/**
 * 统计接口实现
 *
 * @author Lay
 * @date 2020/5/29
 */
@Path("/v1/stat")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class StatFacadeImpl extends BaseFacade implements StatFacade {

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    /**
     * 增加 APP 下载统计
     *
     * @return
     */
    @POST
    @Path("addAppDownload")
    @PermitGuest
    @Override
    public Result addAppDownload() {
        Long channelId = Optional.ofNullable(this.getHeader().getChannelId()).orElse(0L);
        if (channelId < 0) {
            channelId = 0L;
        }

        if (ClientType.iOS == this.getHeader().getClientType()) {
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.DOWNLOAD_COUNT, BigDecimal.ONE);
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.DOWNLOAD_COUNT_IOS, BigDecimal.ONE);
        } else if (ClientType.Android == this.getHeader().getClientType()) {
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.DOWNLOAD_COUNT, BigDecimal.ONE);
            statPlatformService.addPlatformStat(channelId, StatPlatformTypeEnum.DOWNLOAD_COUNT_ANDROID, BigDecimal.ONE);
        }

        return new Result();
    }
}
